@@ -1,9 +1,10 @@
 /*
- * Copyright  2000-2005 The Apache Software Foundation
- *
- *  Licensed under the Apache License, Version 2.0 (the "License");
- *  you may not use this file except in compliance with the License.
- *  You may obtain a copy of the License at
+ *  Licensed to the Apache Software Foundation (ASF) under one or more
+ *  contributor license agreements.  See the NOTICE file distributed with
+ *  this work for additional information regarding copyright ownership.
+ *  The ASF licenses this file to You under the Apache License, Version 2.0
+ *  (the "License"); you may not use this file except in compliance with
+ *  the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -21,9 +22,12 @@ import java.io.File;
 import java.io.IOException;
 import java.util.Enumeration;
 import java.util.Vector;
+import java.util.Locale;
+
 import org.apache.tools.ant.BuildException;
 import org.apache.tools.ant.Project;
 import org.apache.tools.ant.Task;
+import org.apache.tools.ant.taskdefs.condition.Os;
 import org.apache.tools.ant.types.Commandline;
 import org.apache.tools.ant.types.Environment;
 import org.apache.tools.ant.types.Path;
@@ -42,8 +46,10 @@ public class ExecTask extends Task {
     private static final FileUtils FILE_UTILS = FileUtils.getFileUtils();
 
     private String os;
+    private String osFamily;
 
     private File dir;
+    // CheckStyle:VisibilityModifier OFF - bc
     protected boolean failOnError = false;
     protected boolean newEnvironment = false;
     private Long timeout = null;
@@ -65,6 +71,7 @@ public class ExecTask extends Task {
 
     protected Redirector redirector = new Redirector(this);
     protected RedirectorElement redirectorElement;
+    // CheckStyle:VisibilityModifier ON
 
     /**
      * Controls whether the VM (1.3 and above) is used to execute the
@@ -72,6 +79,24 @@ public class ExecTask extends Task {
      */
     private boolean vmLauncher = true;
 
+
+    /**
+     * Create an instance.
+     * Needs to be configured by binding to a project.
+     */
+    public ExecTask() {
+    }
+
+    /**
+     * create an instance that is helping another task.
+     * Project, OwningTarget, TaskName and description are all
+     * pulled out
+     * @param owner task that we belong to
+     */
+    public ExecTask(Task owner) {
+        bindToOwner(owner);
+    }
+
     /**
      * Set whether or not you want the process to be spawned.
      * Default is false.
@@ -135,7 +160,7 @@ public class ExecTask extends Task {
      * @ant.attribute ignore="true"
      */
     public void setCommand(Commandline cmdl) {
-        log("The command attribute is deprecated. "
+        log("The command attribute is deprecated.\n"
             + "Please use the executable attribute and nested arg elements.",
             Project.MSG_WARN);
         this.cmdl = cmdl;
@@ -270,6 +295,7 @@ public class ExecTask extends Task {
     /**
      * Indicates whether to attempt to resolve the executable to a
      * file.
+     * @return the resolveExecutable flag
      *
      * @since Ant 1.6
      */
@@ -362,6 +388,15 @@ public class ExecTask extends Task {
     }
 
 
+    /**
+     * Restrict this execution to a single OS Family
+     * @param osFamily the family to restrict to.
+     */
+    public void setOsFamily(String osFamily) {
+        this.osFamily = osFamily.toLowerCase(Locale.US);
+    }
+
+
     /**
      * The method attempts to figure out where the executable is so that we can feed
      * the full path. We first try basedir, then the exec dir, and then
@@ -398,7 +433,7 @@ public class ExecTask extends Task {
             if (environment != null) {
                 for (int i = 0; i < environment.length; i++) {
                     if (isPath(environment[i])) {
-                        p = new Path(getProject(), environment[i].substring(5));
+                        p = new Path(getProject(), getPath(environment[i]));
                         break;
                     }
                 }
@@ -409,7 +444,7 @@ public class ExecTask extends Task {
                 while (e.hasMoreElements()) {
                     String line = (String) e.nextElement();
                     if (isPath(line)) {
-                        p = new Path(getProject(), line.substring(5));
+                        p = new Path(getProject(), getPath(line));
                         break;
                     }
                 }
@@ -464,12 +499,10 @@ public class ExecTask extends Task {
             throw new BuildException("no executable specified", getLocation());
         }
         if (dir != null && !dir.exists()) {
-            throw new BuildException("The directory you specified does not "
-                                     + "exist");
+            throw new BuildException("The directory " + dir + " does not exist");
         }
         if (dir != null && !dir.isDirectory()) {
-            throw new BuildException("The directory you specified is not a "
-                                     + "directory");
+            throw new BuildException(dir + " is not a directory");
         }
         if (spawn && incompatibleWithSpawn) {
             getProject().log("spawn does not allow attributes related to input, "
@@ -498,21 +531,33 @@ public class ExecTask extends Task {
      * @return boolean.
      * <ul>
      * <li>
-     * <code>true</code> if the os under which Ant is running is
-     * matches one os in the os attribute
-     * or if the os attribute is null</li>
+     * <li><code>true</code> if the os and osfamily attributes are null.</li>
+     * <li><code>true</code> if osfamily is set, and the os family and must match
+     * that of the current OS, according to the logic of
+     * {@link Os#isOs(String, String, String, String)}, and the result of the
+     * <code>os</code> attribute must also evaluate true.
+     * </li>
+     * <li>
+     * <code>true</code> if os is set, and the system.property os.name
+     * is found in the os attribute,</li>
      * <li><code>false</code> otherwise.</li>
      * </ul>
      */
     protected boolean isValidOs() {
-        // test if os match
+        //hand osfamily off to Os class, if set
+        if (osFamily != null && !Os.isOs(osFamily, null, null, null)) {
+            return false;
+        }
+        //the Exec OS check is different from Os.isOs(), which
+        //probes for a specific OS. Instead it searches the os field
+        //for the current os.name
         String myos = System.getProperty("os.name");
         log("Current OS is " + myos, Project.MSG_VERBOSE);
         if ((os != null) && (os.indexOf(myos) < 0)) {
             // this command will be executed only on the specified OS
             log("This OS, " + myos
-                + " was not found in the specified list of valid OSes: " + os,
-                Project.MSG_VERBOSE);
+                    + " was not found in the specified list of valid OSes: " + os,
+                    Project.MSG_VERBOSE);
             return false;
         }
         return true;
@@ -658,7 +703,11 @@ public class ExecTask extends Task {
     }
 
     private boolean isPath(String line) {
-        return line.startsWith("PATH=") || line.startsWith("Path=");
+        return line.startsWith("PATH=")
+            || line.startsWith("Path=");
     }
 
+    private String getPath(String line) {
+        return line.substring("PATH=".length());
+    }
 }
