@@ -1,9 +1,10 @@
 /*
- * Copyright  2003-2005 The Apache Software Foundation
- *
- *  Licensed under the Apache License, Version 2.0 (the "License");
- *  you may not use this file except in compliance with the License.
- *  You may obtain a copy of the License at
+ *  Licensed to the Apache Software Foundation (ASF) under one or more
+ *  contributor license agreements.  See the NOTICE file distributed with
+ *  this work for additional information regarding copyright ownership.
+ *  The ASF licenses this file to You under the Apache License, Version 2.0
+ *  (the "License"); you may not use this file except in compliance with
+ *  the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -20,14 +21,17 @@ import org.apache.tools.ant.AntClassLoader;
 import org.apache.tools.ant.BuildException;
 import org.apache.tools.ant.Project;
 import org.apache.tools.ant.ProjectComponent;
+import org.apache.tools.ant.MagicNames;
 import org.apache.tools.ant.types.Path;
 import org.apache.tools.ant.types.Reference;
 
+// CheckStyle:HideUtilityClassConstructorCheck OFF - bc
+
 /**
  * Offers some helper methods on the Path structure in ant.
  *
- * <p>Basic idea behind this utility class is to use it from inside the
- * different ant objects (and user defined objects) that need classLoading
+ * <p>The basic idea behind this utility class is to use it from inside the
+ * different Ant objects (and user defined objects) that need classLoading
  * for their operation.
  * Normally those would have a setClasspathRef() {for the @classpathref}
  * and/or a createClasspath() {for the nested &lt;classpath&gt;}
@@ -64,11 +68,11 @@ import org.apache.tools.ant.types.Reference;
  * @since Ant 1.6
  */
 public class ClasspathUtils {
-    private static final String LOADER_ID_PREFIX = "ant.loader.";
+
     /**
      * Name of the magic property that controls classloader reuse in Ant 1.4.
      */
-    public static final String REUSE_LOADER_REF = "ant.reuse.loader";
+    public static final String REUSE_LOADER_REF = MagicNames.REFID_CLASSPATH_REUSE_LOADER;
 
     /**
      * Convenience overloaded version of {@link
@@ -91,12 +95,12 @@ public class ClasspathUtils {
      * String, boolean)}.
      *
      * <p>Delegates to the other one after extracting the referenced
-     * Path from the Project This checks also that the passed
+     * Path from the Project. This checks also that the passed
      * Reference is pointing to a Path all right.</p>
-     * @param p current ant project
+     * @param p current Ant project
      * @param ref Reference to Path structure
      * @param reverseLoader if set to true this new loader will take
-     * precedence over it's parent (which is contra the regular
+     * precedence over its parent (which is contra the regular
      * classloader behaviour)
      * @return The class loader
      */
@@ -111,7 +115,7 @@ public class ClasspathUtils {
                     + pathId
                     + " does not reference a Path.");
         }
-        String loaderId = LOADER_ID_PREFIX + pathId;
+        String loaderId = MagicNames.REFID_CLASSPATH_LOADER_PREFIX + pathId;
         return getClassLoaderForPath(p, (Path) path, loaderId, reverseLoader);
     }
 
@@ -121,7 +125,7 @@ public class ClasspathUtils {
      *
      * <p>Assumes the logical 'false' for the reverseLoader.</p>
      *
-     * @param p current ant project
+     * @param p current Ant project
      * @param path the path
      * @param loaderId the loader id string
      * @return The class loader
@@ -143,7 +147,7 @@ public class ClasspathUtils {
      * @param path the path
      * @param loaderId the loader id string
      * @param reverseLoader if set to true this new loader will take
-     * precedence over it's parent (which is contra the regular
+     * precedence over its parent (which is contra the regular
      * classloader behaviour)
      * @return The class loader
      */
@@ -158,14 +162,14 @@ public class ClasspathUtils {
      * defined in the path argument.
      *
      * <p>Based on the setting of the magic property
-     * 'ant.reuse.loader' this will try to reuse the perviously
+     * 'ant.reuse.loader' this will try to reuse the previously
      * created loader with that id, and of course store it there upon
      * creation.</p>
      * @param p             Ant Project where the handled components are living in.
      * @param path          Path object to be used as classpath for this classloader
      * @param loaderId      identification for this Loader,
      * @param reverseLoader if set to true this new loader will take
-     *                      precedence over it's parent (which is contra the regular
+     *                      precedence over its parent (which is contra the regular
      *                      classloader behaviour)
      * @param reuseLoader   if true reuse the loader if it is found
      * @return              ClassLoader that uses the Path as its classpath.
@@ -184,7 +188,6 @@ public class ClasspathUtils {
                 throw new BuildException("The specified loader id " + loaderId
                     + " does not reference a class loader");
             }
-
             cl = (ClassLoader) reusedLoader;
         }
         if (cl == null) {
@@ -193,22 +196,21 @@ public class ClasspathUtils {
                 p.addReference(loaderId, cl);
             }
         }
-
         return cl;
     }
 
     /**
-     * Gets a fresh, different, not used before classloader that uses the
-     * passed path as it's classpath.
+     * Gets a fresh, different, previously unused classloader that uses the
+     * passed path as its classpath.
      *
      * <p>This method completely ignores the ant.reuse.loader magic
      * property and should be used with caution.</p>
      * @param p             Ant Project where the handled components are living in.
      * @param path          the classpath for this loader
      * @param reverseLoader if set to true this new loader will take
-     *                      precedence over it's parent (which is contra the regular
+     *                      precedence over its parent (which is contra the regular
      *                      classloader behaviour)
-     * @return The fresh, different, not used before class loader.
+     * @return The fresh, different, previously unused class loader.
      */
     public static ClassLoader getUniqueClassLoaderForPath(
         Project p,
@@ -219,7 +221,6 @@ public class ClasspathUtils {
             acl.setParentFirst(false);
             acl.addJavaLibraries();
         }
-
         return acl;
     }
 
@@ -227,7 +228,7 @@ public class ClasspathUtils {
      * Creates a fresh object instance of the specified classname.
      *
      * <p> This uses the userDefinedLoader to load the specified class,
-     * and then makes an instance using the default no-argument constructor
+     * and then makes an instance using the default no-argument constructor.
      * </p>
      *
      * @param className the full qualified class name to load.
@@ -235,18 +236,47 @@ public class ClasspathUtils {
      * @return The fresh object instance
      * @throws BuildException when loading or instantiation failed.
      */
+    public static Object newInstance(
+            String className,
+            ClassLoader userDefinedLoader) {
+        return newInstance(className, userDefinedLoader, Object.class);
+    }
+
+    /**
+     * Creates a fresh object instance of the specified classname.
+     *
+     * <p> This uses the userDefinedLoader to load the specified class,
+     * and then makes an instance using the default no-argument constructor.
+     * </p>
+     *
+     * @param className the full qualified class name to load.
+     * @param userDefinedLoader the classloader to use.
+     * @param expectedType the Class that the result should be assignment
+     * compatible with. (No ClassCastException will be thrown in case
+     * the result of this method is casted to the expectedType)
+     * @return The fresh object instance
+     * @throws BuildException when loading or instantiation failed.
+     * @since Ant 1.7
+     */
     public static Object newInstance(
         String className,
-        ClassLoader userDefinedLoader) {
+        ClassLoader userDefinedLoader,
+        Class expectedType) {
         try {
-            Class clazz = userDefinedLoader.loadClass(className);
+            Class clazz = Class.forName(className, true, userDefinedLoader);
             Object o = clazz.newInstance();
+            if (!expectedType.isInstance(o)) {
+                throw new BuildException(
+                    "Class of unexpected Type: "
+                        + className
+                        + " expected :"
+                        + expectedType);
+            }
             return o;
         } catch (ClassNotFoundException e) {
             throw new BuildException(
-                "Class "
-                    + className
-                    + " not found by the specific classLoader.",
+                "Class not found: "
+                    + className,
                 e);
         } catch (InstantiationException e) {
             throw new BuildException(
@@ -262,6 +292,12 @@ public class ClasspathUtils {
                     + ". Specified class should have a "
                     + "public constructor.",
                 e);
+        } catch (LinkageError e) {
+            throw new BuildException(
+                "Class "
+                    + className
+                    + " could not be loaded because of an invalid dependency.",
+                e);
         }
     }
 
@@ -297,8 +333,8 @@ public class ClasspathUtils {
      * <li> attribute @classname </li></ul>
      *
      * <p> This class functions as a delegate handling the configuration
-     * issues for this recuring pattern.  Its usage pattern, as the name
-     * suggests is delegation, not inheritance. </p>
+     * issues for this recurring pattern.  Its usage pattern, as the name
+     * suggests, is delegation rather than inheritance. </p>
      *
      * @since Ant 1.6
      */
@@ -311,7 +347,7 @@ public class ClasspathUtils {
         private boolean reverseLoader = false;
 
         /**
-         * Constructs Delegate
+         * Construct a Delegate
          * @param component the ProjectComponent this delegate is for.
          */
         Delegate(ProjectComponent component) {
@@ -406,14 +442,12 @@ public class ClasspathUtils {
          * @return The class loader.
          */
         public ClassLoader getClassLoader() {
-            ClassLoader cl;
-            cl = ClasspathUtils.getClassLoaderForPath(
+            return getClassLoaderForPath(
                     getContextProject(),
                     this.classpath,
                     getClassLoadId(),
                     this.reverseLoader,
                     loaderId != null || isMagicPropertySet(getContextProject()));
-            return cl;
         }
 
         /**
@@ -428,11 +462,9 @@ public class ClasspathUtils {
          * @return a loader identifier.
          */
         public String getClassLoadId() {
-            if (this.loaderId == null && this.classpathId != null) {
-                return ClasspathUtils.LOADER_ID_PREFIX + this.classpathId;
-            } else {
-                return this.loaderId;
-            }
+            return this.loaderId == null && this.classpathId != null
+                ? MagicNames.REFID_CLASSPATH_LOADER_PREFIX + this.classpathId
+                : this.loaderId;
         }
 
         /**
@@ -442,8 +474,7 @@ public class ClasspathUtils {
          * @return the fresh instantiated object.
          */
         public Object newInstance() {
-            ClassLoader cl = getClassLoader();
-            return ClasspathUtils.newInstance(this.className, cl);
+            return ClasspathUtils.newInstance(this.className, getClassLoader());
         }
 
         /**
